#include <setplay/setplaymanager.h>
#include <setplay/setplayexception.h>

#include <fstream>


using namespace fcportugal::setplay;

// no CBR
SetplayManager::SetplayManager(const string teamName,
			       const string conf_file,const bool useComm,
			       const bool limitedComm,
			       const float messageRepeatTime_,
			       const Context& world)
  :currentSetplay(NULL),
   lastSetplayEndTime(-1),participatedInLastSetplay(false),
   //wasLastLeadPlayer(false),
   lastSetplayWasSuccessful(false),
   messageRepeatTime(messageRepeatTime_),
   teamName(teamName){
	//read conf file
	vector<string> text;
	string line;
	ifstream textstream(conf_file.c_str());
	while (getline(textstream, line)) {
		text.push_back(line);
	}
	textstream.close();
	string alltext;
	for (unsigned int i=0; i < text.size(); i++){
		// substitute tabs by spaces
		size_t j = text[i].find('\t', 0);
		while(j!=string::npos){
			text[i].replace(j,1," ");
			j = text[i].find("\t", 0);
		}
		alltext += text[i];
	}


	string rest;

	Setplay* sp=Setplay::parse(alltext,rest,useComm,limitedComm,
			messageRepeatTime_);

	while(sp){
		if(getSetplay(sp->id())!=NULL)
			cerr<<"SP: duplicate id in setplay file!"<<endl;
		else{
			sp->substituteNamedRegions(world);

			setplays.push_back(sp);

			// Invertible?
			if(sp->invertible)
				setplays.push_back(sp->inversion());
		}

		sp=Setplay::parse(rest,rest,useComm,limitedComm,
				messageRepeatTime_);
	}

	// set random seed
	srand(time(NULL));

	if(rest.length()>0)
    cerr<<"\n Remainder of Setplay Config file:"<<rest<<endl;
  InitializeQLearningMatrices();


//  for(vvd_iterator = M.begin();vvd_iterator!=M.end();++vvd_iterator)
//  {
//      for(vd_iterator = (*vvd_iterator).begin();vd_iterator!=(*vvd_iterator).end();++vd_iterator) {
//       cout<<*vd_iterator<<" ";
//      }
//      cout << endl;
//  }

}
void SetplayManager::InitializeQLearningMatrices()
{
     //const //JAF2014: Loading Q Matrixes for each setplay id loaded...if matrix does not exists, create!
     Setplay *auxSetplay;

      char auxName[4]="";//Will use number as name!!!
      for (int i=0;i<64;i++)//JAF2014: 64 is the maximum number of setplays...id=0 to 63
      {
          auxSetplay = const_cast<Setplay *>(getSetplay(i));
          if(auxSetplay!=NULL)//NULL = this setplay ID was not loaded in setplay.conf file(what about inverted???)
          {
              cout << "SetplayManager::InitializeQLearningMatrices(): Loading matrixQ for setplay " << i << endl;cout.flush();
              //auxSetplay->loadMatrixQ(i);
              MatrixesQ[i] = new QLearningMatrix();
              MatrixesQ[i]->setId(i);
              sprintf(auxName,"   ");
              sprintf(auxName, "%d", i);
              MatrixesQ[i]->setName(auxName);
              //MatrixesQ[i]->setnum_of_states();//JAF dont know, max = 20(MAXSTATES)
              //MatrixesQ[i]->setnum_of_actions();//JAF dont know, max = 20(MAXACTIONS)
              MatrixesQ[i]->loadMatrixQ();
              MatrixesQ[i]->setState(0);//considering that all setplays start on state 0!!!
              cout << "Going to Initiale MatrixQ for setplay " << i << endl;
              //auxSetplay->setMatrixQ(MatrixesQ[i]);// Forwards the pointer to the setplay for update....
              MatrixesQ[i]->showMatrixQ();
     // First time, to create matrixes!!!         MatrixesQ[i]->saveMatrixQ();
          }

     }//JAF2014: End of loading of Qlearning Matrices - only allocated on indices where there is a setplay loaded!!!

}

int SetplayManager::QLearningChooseTransition(const Context& context, vector<const Transition*>* transitions)
{
    int jafdebug=0;
    int MAX_TRANSITIONS = 10;//JAF2014: Maximum number of possible transitions to choose from!
    unsigned int JAFPrioritaryTransitions[MAX_TRANSITIONS];//JAF2014
    //vector<const Transition*>* spTransitions = possibleTransitions(context);

    //JAF2014 - Merging with QLearningMatrix
    //int currentState = currentSetplay->currentStepNumber();
    //MatrixesQ[currentSetplay->id()]->setState(currentState);

    if(transitions->size()>1)
    {
        cout<<"SP2014:SetplayManager::QlearningChooseTransition()::Player " << context.me().number <<  "have " << (transitions?transitions->size():0) //JAF2014
        << " available Transitions in cycle " << context.time()<<endl;         //JAF2014
        jafdebug=1;
    }

    if(jafdebug)
        cout<<"SetplayManager::QlearningChooseTransition(): Setplay in execution:" << currentSetplay->id()<<endl;

    QLearningMatrix *auxQLearningMatrix;
    auxQLearningMatrix=currentMatrixQ;
    if(auxQLearningMatrix==NULL)
        cout << "SetplayManager::QlearningChooseTraining():ERRR!!!QLearningMatrix NULL!!!";
    else
    {  if(jafdebug)
        {
            cout << "going to call currentSetplay->currentMatrixQ->getId() ";
            cout.flush();
            cout << auxQLearningMatrix->getId() << endl;
            cout.flush();
        }
        currentSetplay->setMatrixQ(auxQLearningMatrix);
     }
     if(jafdebug) cout << "Transitions from this State to State(s):";cout.flush();

     Transition *auxTransition;

     for(unsigned int i=0;i!=transitions->size();i++)
     {
         auxTransition =const_cast<Transition *>(transitions->at(i));
         if(auxTransition!=NULL)
          if(auxTransition->isNextStep()||auxTransition->isFinish())
          {
             if(jafdebug)
             {
                 if(dynamic_cast<NextStep*>(auxTransition)) cout << ((NextStep*)auxTransition)->nextStepNumber << "(N) : ";
                 else if(dynamic_cast<Finish*>(auxTransition) ) cout << "(F) : ";
             }
            // Finish *f;
             JAFPrioritaryTransitions[i] =1+
                auxQLearningMatrix->getCellmatrixQ(0,((NextStep*)auxTransition)->nextStepNumber);
          }
     }
     if(jafdebug) {
         cout << endl;cout.flush();
         cout << "JAFPrioritaryTransitions vector = :";

         for(unsigned int j=0;j<transitions->size();j++) //JAF2014: Prints the priority vector JAFAUXPriorityActions
           cout << JAFPrioritaryTransitions[j] << " ";//JAF2014
         cout << endl;cout.flush();
     }

     int chosenTransitionNum;

//Waits until the last moment to select which transition will be executed(time>(currentStepInitTime+abortTime-2)) JAF2014
//     if((context.time())>=(currentSetplay->currentStepInitTime() + currentSetplay->getcurrentStep()->waitTime - 2)
//             ||currentSetplay->getcurrentStep()->waitTime<=2)//waitTimeHasElapsed(context) )
//     {
//         cout << "GOING TO SELECT TRANSITION: currentStepInitTime="<<currentSetplay->currentStepInitTime() <<
//                 " abortTime="<<currentSetplay->getcurrentStep()->waitTime <<
//                 " context.time()="<< context.time() << endl;

//         for(unsigned int i=0; transitions!=NULL && i != transitions->size(); i++)
//             if(transitions->at(i)!=NULL && !isNextStepAlreadyChosen() ){
////                 dlog.addText( Logger::PLAN,//DEBUG
////                           __FILE__"Transition that WAS be Selected: %i",i);
//                 cout << "setplaymanager::QLearningChooseTransition() !!!Transition "<<i<<" WAS selected!!!"<<endl;
//                 chosenTransitionNum=i;//currentSetplay->chosenTransition(i,context);
//                 break; // Ignore remaining transitions...
//             }//end of for+if
         float q_prob = 0.33f;
         float q;
         int count = 0;//JAFROBOCUP2014

         if(transitions->size()>1)
         {
             cout << "Size of Transitions vector = " << transitions->size() << endl;
             q = (rand()%10)/10.0f;

             cout<<"\n"<<"Random Variable q : "<<q<< endl;cout.flush();
             if (q <= q_prob)
             { //choosing random action 1/3 of times....
                 do {
                     chosenTransitionNum = (int)(rand()%transitions->size());
                     //JAFROBOCUP2014 cout<<"\n"<<"searching random valid transition : "<<chosenTransitionNum<<endl;cout.flush();
                     //cout<<"\n"<<matrizR[estado][acao];
                     cout.flush();
                     count++;
                 } while ((transitions->at(chosenTransitionNum)==NULL)&&(count<10));
                 if (count==10) chosenTransitionNum=0;//JAFROBOCUP2014
             }
             cout << "setplaymanager::QLearningChooseTransition() !!!Transition "<<chosenTransitionNum<<" WAS RANDOMLY selected!!!"<<endl;
             //currentSetplay->chosenTransition(chosenTransitionNum,context);
         }//end of if(spTransitions->size()>1) Ramdom choice of transition with probability q_prob
         else
         {
             //cout << "setplaymanager::QLearningChooseTransition() !!!Transition "<<chosenTransitionNum<<" WAS selected!!!"<<endl;
             //currentSetplay->chosenTransition(chosenTransitionNum,context);
             chosenTransitionNum = 0;
         }

//     }//end of if((context.time())>=(currentSetplay->......)
         return chosenTransitionNum;
}

int SetplayManager::QLearningChooseAction(const Context& world)
{
    int jafdebugsetplay = 0;
    int chosenActionNum=0;

    int MAX_ACTIONS = 10;//JAF2014: Maximum number of possible actions to choose from!
    unsigned int JAFPrioritaryActions[MAX_ACTIONS];//JAF2014
    unsigned int JAFAUXPrioritaryActions[MAX_ACTIONS];//JAF2014

    //Check if there are actions to execute
    vector<vector<const fcportugal::setplay::Action* > *> * spActions= currentSetplay->possibleActions(world);

    //JAF2014 - Merging with QLearningMatrix
    int currentState = currentSetplay->currentStepNumber();
    //MatrixesQ[currentSetplay->id()]->setState(currentState);
    Step *auxStep = currentSetplay->getcurrentStep();
    if(spActions->size()>1)
    {
        cout<<"SP2014:SetplayManager::QlearningChooseAction()::Player has "<<(spActions?spActions->size():0) //JAF2014
        << " available actions in cycle " << world.time()<<endl;         //JAF2014
        jafdebugsetplay=1;
    }
    if(jafdebugsetplay)
        cout<<"SetplayManager::QlearningChooseAction(): Setplay in execution:" << currentSetplay->id()<<endl;

    QLearningMatrix *auxQLearningMatrix;
    auxQLearningMatrix=currentMatrixQ;//was!!!!!MatrixesQ[currentSetplay->id()];
    if(auxQLearningMatrix==NULL)
        cout << "SetplayManager::QlearningChooseAction():ERRR!!!QLearningMatrix NULL!!!";
    else
    {  if(jafdebugsetplay)
        {
            cout << "going to call currentSetplay->currentMatrixQ->getId() ";// << currentSetplay->currentMatrixQ->getId();
            cout.flush();
            cout << auxQLearningMatrix->getId() << endl;cout.flush();
        }
        currentSetplay->setMatrixQ(auxQLearningMatrix);
     }
       if(jafdebugsetplay) cout << "Transitions from this State to State(s):";cout.flush();

    Transition *auxTransition;
    for(unsigned int i=0;i!=auxStep->transitions->size();i++)
    {
        auxTransition = auxStep->transitions->at(i);
        if(auxTransition->isNextStep())
        {

            if(jafdebugsetplay) cout << ((NextStep*)auxTransition)->nextStepNumber << " : ";
            JAFPrioritaryActions[i] =1+
//                    auxQLearningMatrix->getCellmatrixQ(currentState,((NextStep*)auxTransition)->nextStepNumber);
            auxQLearningMatrix->getCellmatrixQ(0,((NextStep*)auxTransition)->nextStepNumber);
        }
    }
    if(jafdebugsetplay) {
        cout << endl;cout.flush();
        cout << "JAFPrioritaryAction vector = :";

        for(unsigned int j=0;j<auxStep->transitions->size();j++) //JAF2014: Prints the priority vector JAFAUXPriorityActions
          cout << JAFPrioritaryActions[j] << " ";//JAF2014
        cout << endl;cout.flush();
    }
    //JAF2014: Must choose an action. LMOTA used to choose the first one, or the first not NULL!
    //JAF2014: Now we will try to find the BEST action to choose based on a Reinforcement Learning technique....
    //JAF2014: Starting from a vector with the "priorities" of actions, choose the action of highest priority that
    //JAF2014: is "enabled", that is, which condition was satisfied when constructing the spActions vector!
    //JAF2014: Example: vector of priorities is [2 1 3] (or [120 110 130])
    //JAF2014: where the HIGHER number indicates HIGHER priority: in this case, action 3 should be executed if enabled,
    //JAF2014: if this is not possible choose action 1, and in last case only choose action 2!10/01/2014!

    unsigned int candidate_action_number;//JAF2014
    unsigned int candidate_action_priority=0;//JAF2014
    unsigned int size_Actions_vector = auxStep->transitions->size();//spActions->size();//JAF2014
    bool chosenAction_flag=false;//JAF2014 - boolean that indicates that an ACTION was found, with HIGHER priority
    bool foundCandidate_flag=false;//JAF2014 - boolean that indicates that at least ONE candidate action is still available
    //JAF2014: when all candidate actions where tested, but not even ONE can be choosen, this
    //variable is left set to false, and so we need another option..End the setplay?execute
    // an action even with its condition of execution false??? Dont know.....JAF 10/01/2014.
    chosenActionNum=0;//JAF2014

    for(unsigned int j=0;j<size_Actions_vector;j++)//JAF2014: copy the priority vector JAFPriorityActions to JAFAUXPriorityActions
        JAFAUXPrioritaryActions[j]=JAFPrioritaryActions[j];//JAF2014

    while(!chosenAction_flag){ //JAF2014: while an action has not been choosen do:
        foundCandidate_flag=false;
        if(jafdebugsetplay){           //JAF2014:
            cout << "SP2014:SetplayManager::QlearningChooseAction(): Printing the Vector of Priorities = [ ";
            for(unsigned int k=0;k<size_Actions_vector;k++) cout << JAFAUXPrioritaryActions[k] << " ";
            cout << " ]" << endl;
            fflush(stdout);
        }//JAF2014
        candidate_action_number=0;//JAF2014
        candidate_action_priority=0;//JAF2014
        for(unsigned int i=0;i<size_Actions_vector;i++){//JAF2014:Finds the Action with the HIGHER priority
            if(JAFPrioritaryActions[i]>candidate_action_priority){  //JAF2014
                candidate_action_number = i;                        //JAF2014
                candidate_action_priority = JAFPrioritaryActions[i];//JAF2014
                foundCandidate_flag=true;
            }//JAF2014
        }//JAF2014
        if(foundCandidate_flag==false) break;//JAF2014: no candidate action found, it means all positions of the vector are 0'ed.
        if(spActions->at(candidate_action_number)!=NULL){//JAF2014
            chosenActionNum=candidate_action_number+1;//JAF2014: +1 because the vector is from 0..size_Actions_vector-1
            chosenAction_flag = true;///JAF2014
            if(jafdebugsetplay){           //JAF2014: and the action number choosen varies from 1..size_Actions_vector
                cout << "SP2014:SetplayManager::QlearningChooseAction(): Choosing action: a = "<< chosenActionNum <<
                        "from a vector with " << size_Actions_vector << " possible actions!" << endl;
                fflush(stdout);
            }//JAF2014
        }
        else //JAF2014:spActions->at(candidate_action_number)==NULL, the action cannot be choosen, lets try to find another one!
            JAFPrioritaryActions[candidate_action_number]=0;//JAF2014: discard this option, looks for the next best action!!!
    }//end of while(!choosenActionNum) //JAF2014
    if(foundCandidate_flag)//JAF2014
    {
        chosenActionNum--;//JAF2014: because the vector of Actions is indexed from 0..size_Actions_vector

        if(size_Actions_vector>1)
        {
            cout << "Size Actions vector = " << size_Actions_vector << endl;
            float q = (rand()%10) ;
            q = (float)q /10.0f;

            cout<<"\n"<<"Random Variable q : "<<q<< endl;
            cout.flush();
            if (q <= 0.33f)
            { //escolha uma acao aleatoria 1/3 das vezes....
                do {
                    chosenActionNum = rand()%spActions->size();
                    cout<<"\n"<<"searching random valid action : "<<chosenActionNum;cout.flush();
                    //cout<<"\n"<<matrizR[estado][acao];
                    cout.flush();
                } while (spActions->at(chosenActionNum)==NULL);
            }
        }
    }
    else//JAF2014 !foundcandidate_flag....
    {//JAF2014
        cerr <<"ERRRRRRRRRR!!!! SP2014:SetplayManager::QlearningChooseAction(): No valid ACTION to choose? Executing the first one!!!!" << endl;//JAF2014
        chosenActionNum = 0;//JAF2014
    }//JAF2014

        //      //DEBUG
    if(jafdebugsetplay){ //JAF2013
        cout << "SP2014:SetplayManager::QLearningChooseAction(): Chosen action num: "<<chosenActionNum<<" of "<<(spActions!=NULL?spActions->size():0)<<endl;
        fflush(stdout);
    }
    return(chosenActionNum);
}

SetplayManagerWithCBR::SetplayManagerWithCBR(const string teamName,
					     const string conf_file,
					     const bool useComm,
					     const bool limitedComm,
					     const float messageRepeatTime_,
					     const Context& world,
					     const string cbrFile, 
					     bool writeCBRToFile, 
					     bool writeCBRToFileAfterExecution,
					     const double weightThisGame,
					     const double weightThisTeam,
					     const double weightOtherTeams)
  :SetplayManager(teamName,conf_file,useComm,limitedComm,messageRepeatTime_,
		 world),
  caseBase(new CaseBase(teamName,cbrFile,writeCBRToFile,
			writeCBRToFileAfterExecution,
			weightThisGame,weightThisTeam,
			weightOtherTeams)),
			initialBallPos(NULL),
			gm(NULL){

  // set random seed
  srand(time(NULL));

  //cerr<<"SP: CBR initial case base:"<<endl;//DEBUG
  //caseBase->write(cerr);//DEBUG
}


SetplayManagerWithCBR::~SetplayManagerWithCBR(){
  if(caseBase!=NULL) caseBase->writeToFile();
}

SetplayManager::~SetplayManager(){
  for(unsigned int i=0; i!=  setplays.size();i++)
    delete setplays[i];
}

unsigned int SetplayManager::numberOfSetplays() const{
  return setplays.size();
}

const Setplay* SetplayManager::getSetplay(int num) const{
//JAF2014 Setplay* SetplayManager::getSetplay(int num){
  for(vector<Setplay*>::const_iterator i=setplays.begin();
      i!=setplays.end();i++)
    if((*i)->id()==num)
      return *i;

  return NULL;
}

double  SetplayManager::currentStepInitTime() const{
  if(currentSetplay==NULL ){
	  throw SetplayException("Trying to access setplay init time, but none is active!");
    assert(false);//tanga. Sera' que devia rebentar?
  }

  return currentSetplay->currentStepInitTime();
}

const Action* SetplayManager::currentAction(const Context& world) const{
  assert(currentSetplay);
    
  return currentSetplay->currentAction(world);
}


void SetplayManager::abort(const Context& world){
  assert(currentSetplay);
  currentSetplay->markAsDone();
  dealWithSetplayEnd(world);
}

void SetplayManagerWithCBR::abort(const Context& world){
  assert(currentSetplay);
  currentSetplay->markAsDone();
  
  dealWithSetplayEnd(world);
}




void SetplayManager::finish(const Context& world){
  assert(currentSetplay);
  currentSetplay->markAsDone();
  currentSetplay->markAsSuccess();
  dealWithSetplayEnd(world);

}


void SetplayManagerWithCBR::finish(const Context& world){
  assert(currentSetplay);
  currentSetplay->markAsDone();
  currentSetplay->markAsSuccess();
  
  dealWithSetplayEnd(world);
 
}

bool SetplayManager::isDone() const{
  assert(currentSetplay);

  return currentSetplay->isDone();
}

void SetplayManager::writeSetplayNum(ostream &os,
				     const unsigned int setplayNumber) const{
  if(setplayNumber>=setplays.size()){
    cerr<<"SP: inexistent Setplay in write number:"<<setplayNumber<<endl;
    return;
  }
  setplays[setplayNumber]->write(os);
}

void SetplayManager::writeSetplayById(ostream &os,
				  const int id) const{
  if(getSetplay(id)==NULL){
    cerr<<"SP: inexistent Setplay in write with id:"<<id<<endl;
    return;
  }
  getSetplay(id)->write(os);
}

void SetplayManager::writeCurrentSetplay(ostream &os) const{
  assert(currentSetplay);

  currentSetplay->write(os);
}



void SetplayManager::substituteNamedRegions(const Context& world){
  for(unsigned int i=0; i!=  setplays.size();i++)
    setplays[i]->substituteNamedRegions(world);
}

unsigned int
SetplayManagerWithCBR::selectSetplayByCBR(vector<CaseBase::SetplayID> possibleSetplays,
					  const Context& world) const{
  cerr<<"SP: choosing SP by CBR: caseBase null:"<<(caseBase==NULL)
      <<" number of possible SPs:"<<possibleSetplays.size()
      <<endl;
  // CBR select
  int chosenSPId=caseBase->selectSetplay(possibleSetplays,world);
  
  cerr<<"SP: chosen setplay has ID: "<<chosenSPId<<endl;//DEBUG
  
  return chosenSPId;
}

void SetplayManagerWithCBR::autoActivateSetplay(const Context& world){
  // get all valid setplays
  vector<const Setplay*>* feasibleSPs=feasibleSetplays(world);
//  cerr << "SP:autoActivateSetplay():";//JAF2014
  // exit if no Setplay is available
  if(feasibleSPs->empty()){
//      cerr << "Empty set of Feasible Setplays!" << endl;//JAF2014
      return;
  }
  // Create vector with possible setplay names
  vector<CaseBase::SetplayID>feasibleSPIds;
  for(unsigned int i=0;feasibleSPs!=NULL && i!=feasibleSPs->size();i++)
    feasibleSPIds.push_back(CaseBase::SetplayID(feasibleSPs->at(i)->name,
						feasibleSPs->at(i)->id()));

  // CBR select
  int chosenSPId=selectSetplayByCBR(feasibleSPIds,world);

  // Check if no Setplay could be chosen
  if(chosenSPId==10000)
    return;

  const fcportugal::setplay::Setplay* chosenSetplay
    = getSetplay(chosenSPId);
  
  // Choose participating players
  vector<unsigned int>* participatingPlayers
    = chosenSetplay->participatingPlayersDistanceFromPositions(world);

//  cerr<<"SP: chosen players:";//DEBUG
//  for(unsigned int i=0;i!=participatingPlayers->size();i++)//DEBUG
//	  cerr<<participatingPlayers->at(i)<<endl;//DEBUG

  // Activate the setplay with chosen players
  // No other arguments for the time being
  activateSetplay(InitMessage(world.time(),
			      chosenSetplay->id(),
			      participatingPlayers),world);
  
  cerr<<"SP:autoActivateSetplay():Init:"<<messageToSend(world)->toString()<<endl;//DEBUG

  // for CBR, register start data
  initialBallPos=world.ballPos();
  gm=new const rcss::clang::PlayMode(world.playMode());
  cerr << "SP:autoActivateSetplay(): PlayMode = " << world.playMode() << endl;
}

void SetplayManagerWithCBR::activateSetplay(const InitMessage init,
					    const Context& world){
  SetplayManager::activateSetplay(init,world);

  // for CBR, register start data
  initialBallPos=world.ballPos();
  gm=new const rcss::clang::PlayMode(world.playMode());

}

void SetplayManager::activateSetplay(const InitMessage init,
				     const Context& world){

  Setplay* intendedSetplay
    =(getSetplay(init.setplayNumber)!=NULL?
      getSetplay(init.setplayNumber)->deepCopy():NULL);

  bool instantiated 
    =(intendedSetplay!=NULL?intendedSetplay->instantiate(init):false);

  if(!instantiated){
    cerr<<world.me().number<<" failed to instantiate setplay "<<init.setplayNumber<<"!"<<endl;//DEBUG
    return;
  }


  // Setplay is always kept, even if  player does not participate
  // This is useful to know the participating players' positions.
  intendedSetplay->start(world);
  currentSetplay=intendedSetplay;
  

  //JAF2014 - Merging with QLearningMatrix
  cout << "SetplayManager::activateSetplay(): Setting MatrixQ for current Setplay number:" << abs(intendedSetplay->id()) << endl;
  if(MatrixesQ[abs(intendedSetplay->id())]==NULL) cout << "Mas a Matriz é NULL!!!!!" << endl;
  cout.flush();
  currentMatrixQ = MatrixesQ[abs(intendedSetplay->id())];//JAF2014 setting current MatrixQ!!!
  currentMatrixQ->setState(0);
  currentSetplay->setMatrixQ(currentMatrixQ);
  //DEBUG
  //cout<<"\nSP:activateSetplay acabou"<<endl;
  
}




vector<const Setplay*>* 
SetplayManager::feasibleSetplays(const Context& world) const{
  vector<const Setplay*>* res= new vector<const Setplay*>();
//  cerr << "SP:feasibleSetplays():Evaluating Conditions.." << endl;//JAF2014
  for(unsigned int i = 0; i!= setplays.size(); i++)
    if(setplays[i]->steps->at(0)->condition==NULL // No condition, always valid
       ||  setplays[i]->steps->at(0)->condition->eval(world))
      res->push_back(setplays[i]);
  if(res->size()>0){//JAF2014
      cerr << "SP:feasibleSetplays found:" << res->size() << endl;//JAF2014
      const Setplay *tmp;//JAF2014
      for(int i=0;i<res->size();i++){
          tmp = res->at(i);//JAF2014
          cout << "SP: feasibleSetplay Number: " << tmp->id() << endl;//JAF2014
      }
  }//JAF2014
  return res;
}


bool SetplayManager::isSetplayActive() const{
  return currentSetplay!=NULL;
}


bool SetplayManager::thisPlayerParticipates(const Context& world) const{
  return (currentSetplay!=NULL &&
	  currentSetplay->playerParticipates(&(world.me())));
}

bool SetplayManager::playerParticipates(const PlayerID* pl) const{
  return currentSetplay && currentSetplay->playerParticipates(pl);
}

bool SetplayManager::amLeadPlayer(const Context& world) const{
  return currentSetplay!=NULL && currentSetplay->amLeadPlayer(world);
}

const PlayerID* SetplayManager::leadPlayer() const{
  assert(currentSetplay!=NULL);

  return currentSetplay->leadPlayer();
}


const PlayerID* SetplayManager::nextStepLeadPlayer() const{

  assert(currentSetplay!=NULL);

  return currentSetplay->nextStepLeadPlayer();
}
bool SetplayManager::waitTimeHasElapsed(const Context& world) const{
  assert(currentSetplay);

  return currentSetplay->waitTimeHasElapsed(world);
}

bool SetplayManager::isNextStepAlreadyChosen() const{
  assert(currentSetplay);

  return currentSetplay->isNextStepAlreadyChosen();
}

vector<vector<const Action*>*>* 
SetplayManager::possibleActions(const Context& world) const{
  if(currentSetplay!=NULL) {
    return currentSetplay->possibleActions(world);
  }
  throw SetplayException("Trying to check possible actions in Setplay, but none is active!");
  return NULL;
}

vector<const Transition*>*
SetplayManager::possibleTransitions(const Context& world) const{
  if(currentSetplay!=NULL)
    return currentSetplay->possibleTransitions(world);
  else
	  throw SetplayException("Trying to check possible Transitions in Setplay, but none is active!");
  return NULL;
}

//considering the chosenActionNumber argument, first action should be 0
void SetplayManager::chosenAction(const unsigned int& chosenActionNumber,
		  const Context& world){
  if(currentSetplay!=NULL)
    currentSetplay->chosenAction(chosenActionNumber,world);
  else
    // Launch exception
    cerr<<"Trying to choose action in Setplay, but none is active!"<<endl;

  //Check if setplay is over...
  if(currentSetplay->isDone()){
    cout<<"SP: "<<world.me().number<<" setplay acabou"<<endl;//DEBUG

    dealWithSetplayEnd(world); 
  }
}

//considering the chosenTransitionNumber argument, 
// first trantition should be 0
void 
SetplayManager::chosenTransition(const unsigned int& chosenTransitionNumber,
				 const Context& world){
  if(currentSetplay!=NULL)

    currentSetplay->chosenTransition(chosenTransitionNumber,world);
  else
    // Launch exception
    cerr<<"Trying to choose action in Setplay, but none is active!"<<endl;

  //Check if setplay is over...
  if(currentSetplay->isDone()){
    cout<<"SP: "<<world.me().number<<" setplay acabou"<<endl;//DEBUG

    dealWithSetplayEnd(world); 
  }
}



void SetplayManager::dealWithSetplayEnd(const Context& world){
  cout<<"SP: "<<world.me().number<<" setplay ended at "<<world.time()<<endl;//DEBUG
  
  lastSetplayEndTime=world.time();
  if(currentSetplay->playerParticipates(&(world.me()))) participatedInLastSetplay=true;
  //if(currentSetplay->amLeadPlayer()) wasLastLeadPlayer=true;
  lastSetplayWasSuccessful=currentSetplay->wasSuccessful();
  delete  currentSetplay;
  currentSetplay=NULL;
}


void SetplayManagerWithCBR::dealWithSetplayEnd(const Context& world){
  // register CBR
  if(caseBase!=NULL){
    cout<<"SP: "<<world.me().number<<" setplay with CBR ended: Going to registerCase: " << endl;//DEBUG
    caseBase->registerCase(currentSetplay->name,
			   initialBallPos,
			   *gm,currentSetplay->wasSuccessful(),
			   world);
    if(currentSetplay!=NULL)
    {
    cout<<"SP: "<<world.me().number<<" setplay with CBR ended. Success: "//DEBUG
	<<currentSetplay->wasSuccessful()<<" at "<<world.time()//DEBUG
	<<" Setplay active?:"<<(isSetplayActive()?"yes":"no")//DEBUG
	<<" lead: "<<currentSetplay->amLeadPlayer(world)<<endl;//DEBUG
    }
  }
  cout<<"SP: "<<world.me().number<<" setplay with CBR ended: Going to finish things...." << endl;//DEBUG
  lastSetplayEndTime=world.time();
  lastSetplayWasSuccessful=currentSetplay->wasSuccessful();
  if(currentSetplay->playerParticipates(&(world.me()))) participatedInLastSetplay=true;
  //if(currentSetplay->amLeadPlayer()) wasLastLeadPlayer=true;
  delete  currentSetplay;
  currentSetplay=NULL;
  cout<<"SP: "<<world.me().number<<" setplay with CBR ended: dealWithSeplayEnd() finished OK!" << endl;//DEBUG
}

void 
SetplayManager::updateInternalState(const Context& world){
//  cerr<<"SP: In SetplayManager::updateInternalState!"<<endl;//DEBUG

  if(currentSetplay!=NULL)
    currentSetplay->updateInternalState(world);
  else
    // Launch exception
    cerr<<"Trying to update internal state in Setplay, but none is active!"<<endl;
  cerr.flush();
  
  //Check if setplay is over...
  if(currentSetplay->isDone())
    dealWithSetplayEnd(world); 
}

//----------------------------------  Communication--------------------------
bool SetplayManager::willCommunicate(const Context& world) const{
  if(currentSetplay!=NULL)
    return currentSetplay->willCommunicate(world); 
  else {//Setplay is over
    // Every player will communicate, to broadcast this important information

    //if (//lastSetplayEndTime!=-1 && me->number >6)//
    //	wasLastLeadPlayer)

      // Check if repetition is already above messageRepeatTime
    if((world.time()-lastSetplayEndTime)>=messageRepeatTime){
      lastSetplayEndTime=-1; //No more repetitions
      lastSetplayWasSuccessful=false;  
      participatedInLastSetplay=false;
      //wasLastLeadPlayer=false;
      
      return false;
    }
    
    // if(lastSetplayEndTime!=-1  && participatedInLastSetplay//DEBUG
    //   && (world.time()-lastSetplayEndTime)<messageRepeatTime)//DEBUG
    //   cerr<<"SP: testando envio de mensagem depois de SP acabada. Era ("//DEBUG
    // 	  <<me->number<<") paticipated: "//DEBUG
    // 	  <<participatedInLastSetplay<<" Tempo atual:"//DEBUG
    // 	  <<world.time()<<" Tempo de fim:"<<lastSetplayEndTime//DEBUG
    // 	  <<" Repeticao:"<<messageRepeatTime<<endl;//DEBUG
    
    // Only participants in last Setplay will communicate Setplay end
    // This avoids messages from players that might be away from the action,
    // and therefore do not hear messages and easily let abort times elapse
    return //wasLastLeadPlayer && 
      lastSetplayEndTime!=-1 && participatedInLastSetplay
      && (world.time()-lastSetplayEndTime)<messageRepeatTime;
  }
}


SetplayMessage* SetplayManager::messageToSend(const Context& world){
  if(currentSetplay!=NULL && !currentSetplay->isDone())
    return currentSetplay->messageToSend(world);
  else
    return new StepMessage(lastSetplayEndTime,-1,lastSetplayWasSuccessful?1:-1);
}


void
SetplayManager::processReceivedMessage(const InitMessage& m,
				       const Context& world){
  if(currentSetplay!=NULL){//A setplay is active!
    if(currentSetplay->id()!= m.setplayNumber){
      cerr<<"SP:An active setplay will be substituted by a different one"<<endl;//DEBUG
      dealWithSetplayEnd(world);
    }
    else if(currentSetplay->currentStepNumber()!=0 && !amLeadPlayer(world)){
      cerr<<"SP: ("<<world.me().number//DEBUG
    		  <<") An active Setplay will be restarted at"<<world.time()//DEBUG
    		  <<endl;//DEBUG
      dealWithSetplayEnd(world);
    }
    else
      return; // Setplay is still in step 0, do nothing    
  }
  else
    activateSetplay(m,world);
}

void
SetplayManager::processReceivedMessage(const StepMessage& m,
				       const Context& world){

//  if(currentSetplay!=NULL && currentSetplay->playerParticipates(&(world.me())))//DEBUG
//    cerr<<"SP: "<<world.me().number<<"recebi mensagem de step:"//DEBUG
//    <<m.toString()<<endl;//DEBUG

  if(currentSetplay!=NULL)
    currentSetplay->processReceivedMessage(m,world);
  else // Check for success in done setplay 
    if(m.currentStepNumber==-1 && m.nextStepNumber==1)
      lastSetplayWasSuccessful=true;
}

// CAUTION is this necessary?
// void
// SetplayManager::processReceivedMessage(const SetplayMessage* m,
// 				       const Context& world){
//   if(m->isInit())
//     processReceivedMessage((InitMessage*)m,world);
//   else
//     processReceivedMessage((StepMessage*)m,world);
// }


void SetplayManager::processReceivedMessage(const string m,
					    const Context& world){
  //cerr<<"SP: recebi mensagem geral:"<<m<<"<<"<<endl;cerr.flush();//DEBUG
  if(m[0]=='S'){//InitMessage
    InitMessage* msg= new InitMessage(world.time(),m);
    processReceivedMessage(*msg,world);
  }else{
    StepMessage* msg= new StepMessage(world.time(),m);
    processReceivedMessage(*msg,world);
  }
}

